import re

from checkov.common.models.consts import DOCKER_IMAGE_REGEX
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check


class ImageTagFixed(BaseK8Check):

    def __init__(self):
        """
        You should avoid using the :latest tag when deploying containers in production
        as it is harder to track which version of the image is running
        and more difficult to roll back properly.
        """
        name = "Image Tag should be fixed - not latest or blank"
        id = "CKV_K8S_14"
        # Location: container .image
        supported_kind = ['containers', 'initContainers']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_resource_id(self, conf):
        return f'{conf["parent"]} - {conf["name"]}'

    def scan_spec_conf(self, conf):
        if "image" in conf:

            image_val = conf["image"]
            if not isinstance(image_val, str) or image_val.strip() == '':
                return CheckResult.UNKNOWN

            # If there's a digest, then this is even better than the tag, so the check passes
            if '@' in image_val:
                return CheckResult.PASSED

            (image, tag) = re.findall(DOCKER_IMAGE_REGEX, image_val)[0]
            if tag == "latest" or tag == "":
                return CheckResult.FAILED
        else:
            return CheckResult.FAILED
        return CheckResult.PASSED


check = ImageTagFixed()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class RedshiftClusterLogging(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Redshift Cluster logging is enabled"
        id = "CKV_AWS_71"
        supported_resource = ['aws_redshift_cluster']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resource)

    def get_inspected_key(self):
        return 'logging/[0]/enable'

check = RedshiftClusterLogging()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AzureManagedDiscEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Azure managed disk have encryption enabled"
        id = "CKV_AZURE_2"
        supported_resources = ['azurerm_managed_disk']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for password configuration at azure_instance:
            https://www.terraform.io/docs/providers/azure/r/instance.html
        :param conf: azure_instance configuration
        :return: <CheckResult>
        """
        if 'encryption_settings' in conf.keys():
            config = conf['encryption_settings'][0]
            if isinstance(config, dict) and not config['enabled'][0]:
                return CheckResult.FAILED
        return CheckResult.PASSED


check = AzureManagedDiscEncryption()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class GoogleProjectDefaultNetwork(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that the default network does not exist in a project"
        id = "CKV_GCP_27"
        supported_resources = ['google_project']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        """
        https://www.terraform.io/docs/providers/google/r/google_project.html
        :param conf: google_project configuration
        :return: <CheckResult>
        """
        return 'auto_create_network/[0]'

    def get_expected_value(self):
        return False


check = GoogleProjectDefaultNetwork()
